@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  GDAL algorithms
  * Purpose:  Test alg
@@ -8,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2016, Even Rouault <even.rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include <array>
@@ -151,6 +134,37 @@ TEST_F(test_alg, GDALWarpResolveWorkingDataType_padfSrcNoDataReal)
     GDALDestroyWarpOptions(psOptions);
 }
 
+// GDALWarpResolveWorkingDataType: effect of padfSrcNoDataReal
+TEST_F(test_alg, GDALWarpResolveWorkingDataType_padfSrcNoDataReal_with_band)
+{
+    GDALDatasetUniquePtr poDS(GDALDriver::FromHandle(GDALGetDriverByName("MEM"))
+                                  ->Create("", 1, 1, 1, GDT_Byte, nullptr));
+    GDALWarpOptions *psOptions = GDALCreateWarpOptions();
+    // False-positive: hSrcDS is no longer used after GDALDestroyWarpOptions()
+    // coverity[escape]
+    psOptions->hSrcDS = GDALDataset::ToHandle(poDS.get());
+    psOptions->nBandCount = 1;
+    psOptions->panSrcBands =
+        static_cast<int *>(CPLMalloc(psOptions->nBandCount * sizeof(int)));
+    psOptions->panSrcBands[0] = 1;
+    psOptions->padfSrcNoDataReal =
+        static_cast<double *>(CPLMalloc(sizeof(double)));
+    psOptions->padfSrcNoDataReal[0] = 0.0;
+    GDALWarpResolveWorkingDataType(psOptions);
+    EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
+
+    psOptions->padfSrcNoDataReal[0] = -1.0;
+    GDALWarpResolveWorkingDataType(psOptions);
+    EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
+
+    psOptions->eWorkingDataType = GDT_Unknown;
+    psOptions->padfSrcNoDataReal[0] = 2.0;
+    GDALWarpResolveWorkingDataType(psOptions);
+    EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
+
+    GDALDestroyWarpOptions(psOptions);
+}
+
 // GDALWarpResolveWorkingDataType: effect of padfSrcNoDataImag
 TEST_F(test_alg, GDALWarpResolveWorkingDataType_padfSrcNoDataImag)
 {
