@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  GDAL algorithms
  * Purpose:  Tests for the marching squares algorithm
@@ -8,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2018, Hugo Mercier, <hugo dot mercier at oslandia dot com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "gdal_unit_test.h"
@@ -34,6 +17,7 @@
 #include "marching_squares/point.h"
 #include "marching_squares/level_generator.h"
 #include "marching_squares/contour_generator.h"
+#include <limits>
 #include <map>
 #include <fstream>
 
@@ -139,7 +123,8 @@ TEST_F(test_ms_tile, dummy)
     // only one pixel of value 2.0
     // levels = 0, 10
     std::vector<double> data = {2.0};
-    IntervalLevelRangeIterator levels(0.0, 10.0);
+    IntervalLevelRangeIterator levels(0.0, 10.0,
+                                      -std::numeric_limits<double>::infinity());
     Writer writer;
 
     ContourGenerator<Writer, IntervalLevelRangeIterator> cg(
@@ -173,7 +158,9 @@ TEST_F(test_ms_tile, tile_one_pixel)
     // levels = 0, 10
     std::vector<double> data = {2.0};
     const double levels[] = {0.0};
-    FixedLevelRangeIterator levelGenerator(levels, 1);
+    FixedLevelRangeIterator levelGenerator(
+        levels, 1, -std::numeric_limits<double>::infinity(),
+        std::numeric_limits<double>::infinity());
     Writer writer;
 
     ContourGenerator<Writer, FixedLevelRangeIterator> cg(
@@ -207,7 +194,8 @@ TEST_F(test_ms_tile, tile_one_pixel_two)
     // only one pixel of value 2.0
     // levels = 2, 10
     std::vector<double> data = {2.0};
-    IntervalLevelRangeIterator levels(2.0, 10.0);
+    IntervalLevelRangeIterator levels(2.0, 10.0,
+                                      -std::numeric_limits<double>::infinity());
     Writer writer;
 
     ContourGenerator<Writer, IntervalLevelRangeIterator> cg(
@@ -299,7 +287,8 @@ TEST_F(test_ms_tile, tile_two_pixels)
 
     std::vector<double> data = {10.0, 7.0};
     {
-        IntervalLevelRangeIterator levels(8.0, 10.0);
+        IntervalLevelRangeIterator levels(
+            8.0, 10.0, -std::numeric_limits<double>::infinity());
         Writer writer;
         ContourGenerator<Writer, IntervalLevelRangeIterator> cg(
             2, 1, /* hasNoData */ false, NaN, writer, levels);
@@ -421,7 +410,8 @@ TEST_F(test_ms_tile, tile_four_pixels)
     //  NaN                 NaN                NaN                NaN
     std::vector<double> data = {10.0, 7.0, 4.0, 5.0};
     {
-        IntervalLevelRangeIterator levels(8.0, 10.0);
+        IntervalLevelRangeIterator levels(
+            8.0, 10.0, -std::numeric_limits<double>::infinity());
         Writer writer;
         ContourGenerator<Writer, IntervalLevelRangeIterator> cg(
             2, 2, /* hasNoData */ false, NaN, writer, levels);
@@ -494,7 +484,9 @@ TEST_F(test_ms_tile, tile_four_pixels_2)
     std::vector<double> data = {155.0, 155.01, 154.99, 155.0};
     {
         const double levels[] = {155.0};
-        FixedLevelRangeIterator levelGenerator(levels, 1);
+        FixedLevelRangeIterator levelGenerator(
+            levels, 1, -std::numeric_limits<double>::infinity(),
+            std::numeric_limits<double>::infinity());
         Writer writer;
         ContourGenerator<Writer, FixedLevelRangeIterator> cg(
             2, 2, /* hasNoData */ false, NaN, writer, levelGenerator);
