@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  GDAL Core
  * Purpose:  Test GDALCopyWords().
@@ -8,26 +7,11 @@
  ******************************************************************************
  * Copyright (c) 2009-2011, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_conv.h"
+#include "cpl_float.h"
 #include "gdal.h"
 
 #include <cstdint>
@@ -175,6 +159,9 @@ class TestCopyWords : public ::testing::Test
         else if (outtype == GDT_UInt64)
             Test<InType, std::uint64_t, ConstantType>(
                 intype, inval, invali, outtype, outval, outvali, numLine);
+        else if (outtype == GDT_Float16)
+            Test<InType, GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                                 outval, outvali, numLine);
         else if (outtype == GDT_Float32)
             Test<InType, float, ConstantType>(intype, inval, invali, outtype,
                                               outval, outvali, numLine);
@@ -187,6 +174,9 @@ class TestCopyWords : public ::testing::Test
         else if (outtype == GDT_CInt32)
             Test<InType, GInt32, ConstantType>(intype, inval, invali, outtype,
                                                outval, outvali, numLine);
+        else if (outtype == GDT_CFloat16)
+            Test<InType, GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                                 outval, outvali, numLine);
         else if (outtype == GDT_CFloat32)
             Test<InType, float, ConstantType>(intype, inval, invali, outtype,
                                               outval, outvali, numLine);
@@ -224,6 +214,9 @@ class TestCopyWords : public ::testing::Test
         else if (intype == GDT_UInt64)
             FromR_2<std::uint64_t, ConstantType>(intype, inval, invali, outtype,
                                                  outval, outvali, numLine);
+        else if (intype == GDT_Float16)
+            FromR_2<GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                            outval, outvali, numLine);
         else if (intype == GDT_Float32)
             FromR_2<float, ConstantType>(intype, inval, invali, outtype, outval,
                                          outvali, numLine);
@@ -236,6 +229,9 @@ class TestCopyWords : public ::testing::Test
         else if (intype == GDT_CInt32)
             FromR_2<GInt32, ConstantType>(intype, inval, invali, outtype,
                                           outval, outvali, numLine);
+        else if (intype == GDT_CFloat16)
+            FromR_2<GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                            outval, outvali, numLine);
         else if (intype == GDT_CFloat32)
             FromR_2<float, ConstantType>(intype, inval, invali, outtype, outval,
                                          outvali, numLine);
@@ -258,8 +254,8 @@ class TestCopyWords : public ::testing::Test
 #define IS_UNSIGNED(x)                                                         \
     (x == GDT_Byte || x == GDT_UInt16 || x == GDT_UInt32 || x == GDT_UInt64)
 #define IS_FLOAT(x)                                                            \
-    (x == GDT_Float32 || x == GDT_Float64 || x == GDT_CFloat32 ||              \
-     x == GDT_CFloat64)
+    (x == GDT_Float16 || x == GDT_Float32 || x == GDT_Float64 ||               \
+     x == GDT_CFloat16 || x == GDT_CFloat32 || x == GDT_CFloat64)
 
 #define CST_3000000000 (((GIntBig)3000) * 1000 * 1000)
 #define CST_5000000000 (((GIntBig)5000) * 1000 * 1000)
@@ -307,10 +303,12 @@ TEST_F(TestCopyWords, GDT_Int8)
     FROM_R(GDT_Int8, -128, GDT_UInt32, 0); /* clamp */
     FROM_R(GDT_Int8, -128, GDT_Int64, -128);
     FROM_R(GDT_Int8, -128, GDT_UInt64, 0); /* clamp */
+    FROM_R(GDT_Int8, -128, GDT_Float16, -128);
     FROM_R(GDT_Int8, -128, GDT_Float32, -128);
     FROM_R(GDT_Int8, -128, GDT_Float64, -128);
     FROM_R(GDT_Int8, -128, GDT_CInt16, -128);
     FROM_R(GDT_Int8, -128, GDT_CInt32, -128);
+    FROM_R(GDT_Int8, -128, GDT_CFloat16, -128);
     FROM_R(GDT_Int8, -128, GDT_CFloat32, -128);
     FROM_R(GDT_Int8, -128, GDT_CFloat64, -128);
     for (GDALDataType outtype = GDT_Byte; outtype < GDT_TypeCount;
@@ -524,6 +522,144 @@ TEST_F(TestCopyWords, GDT_UInt64)
     FROM_R(GDT_UInt64, nVal, GDT_CFloat64, nVal);
 }
 
+TEST_F(TestCopyWords, GDT_Float16only)
+{
+    GDALDataType intype = GDT_Float16;
+    for (GDALDataType outtype = GDT_Byte; outtype < GDT_TypeCount;
+         outtype = (GDALDataType)(outtype + 1))
+    {
+        if (IS_FLOAT(outtype))
+        {
+            FROM_R_F(intype, 127.1, outtype, 127.1);
+            FROM_R_F(intype, -127.1, outtype, -127.1);
+        }
+        else
+        {
+            FROM_R_F(intype, 125.1, outtype, 125);
+            FROM_R_F(intype, 125.9, outtype, 126);
+
+            FROM_R_F(intype, 0.4, outtype, 0);
+            FROM_R_F(intype, 0.5, outtype,
+                     1); /* We could argue how to do this rounding */
+            FROM_R_F(intype, 0.6, outtype, 1);
+            FROM_R_F(intype, 126.5, outtype,
+                     127); /* We could argue how to do this rounding */
+
+            if (!IS_UNSIGNED(outtype))
+            {
+                FROM_R_F(intype, -125.9, outtype, -126);
+                FROM_R_F(intype, -127.1, outtype, -127);
+
+                FROM_R_F(intype, -0.4, outtype, 0);
+                FROM_R_F(intype, -0.5, outtype,
+                         -1); /* We could argue how to do this rounding */
+                FROM_R_F(intype, -0.6, outtype, -1);
+                FROM_R_F(intype, -127.5, outtype,
+                         -128); /* We could argue how to do this rounding */
+            }
+        }
+    }
+    FROM_R(intype, -30000, GDT_Byte, 0);
+    FROM_R(intype, -32768, GDT_Byte, 0);
+    FROM_R(intype, -1, GDT_Byte, 0);
+    FROM_R(intype, 256, GDT_Byte, 255);
+    FROM_R(intype, 30000, GDT_Byte, 255);
+    FROM_R(intype, -330000, GDT_Int16, -32768);
+    FROM_R(intype, -33000, GDT_Int16, -32768);
+    FROM_R(intype, 33000, GDT_Int16, 32767);
+    FROM_R(intype, -33000, GDT_UInt16, 0);
+    FROM_R(intype, -1, GDT_UInt16, 0);
+    FROM_R(intype, 60000, GDT_UInt16, 60000);
+    FROM_R(intype, -33000, GDT_Int32, -32992);
+    FROM_R(intype, 33000, GDT_Int32, 32992);
+    FROM_R(intype, -1, GDT_UInt32, 0);
+    FROM_R(intype, 60000, GDT_UInt32, 60000U);
+    FROM_R(intype, 33000, GDT_Float32, 32992);
+    FROM_R(intype, -33000, GDT_Float32, -32992);
+    FROM_R(intype, 33000, GDT_Float64, 32992);
+    FROM_R(intype, -33000, GDT_Float64, -32992);
+    FROM_R(intype, -33000, GDT_CInt16, -32768);
+    FROM_R(intype, 33000, GDT_CInt16, 32767);
+    FROM_R(intype, -33000, GDT_CInt32, -32992);
+    FROM_R(intype, 33000, GDT_CInt32, 32992);
+    FROM_R(intype, 33000, GDT_CFloat32, 32992);
+    FROM_R(intype, -33000, GDT_CFloat32, -32992);
+    FROM_R(intype, 33000, GDT_CFloat64, 32992);
+    FROM_R(intype, -33000, GDT_CFloat64, -32992);
+
+    // Float16 to Int64
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::quiet_NaN();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::infinity();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, INT64_MIN);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::max();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, -65504);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::max();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, 65504);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::infinity();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, INT64_MAX);
+    }
+
+    // Float16 to UInt64
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::quiet_NaN();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::infinity();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::max();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::max();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 65504);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::infinity();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, UINT64_MAX);
+    }
+}
+
 TEST_F(TestCopyWords, GDT_Float32and64)
 {
     /* GDT_Float32 and GDT_Float64 */
@@ -598,35 +734,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float32 to Int64
     {
-        float in_value = std::numeric_limits<float>::quiet_NaN();
+        float in_value = cpl::NumericLimits<float>::quiet_NaN();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::infinity();
+        float in_value = -cpl::NumericLimits<float>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::max();
+        float in_value = -cpl::NumericLimits<float>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        float in_value = std::numeric_limits<float>::max();
+        float in_value = cpl::NumericLimits<float>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
     }
 
     {
-        float in_value = std::numeric_limits<float>::infinity();
+        float in_value = cpl::NumericLimits<float>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
@@ -634,35 +770,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float64 to Int64
     {
-        double in_value = std::numeric_limits<double>::quiet_NaN();
+        double in_value = cpl::NumericLimits<double>::quiet_NaN();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::infinity();
+        double in_value = -cpl::NumericLimits<double>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::max();
+        double in_value = -cpl::NumericLimits<double>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        double in_value = std::numeric_limits<double>::max();
+        double in_value = cpl::NumericLimits<double>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
     }
 
     {
-        double in_value = std::numeric_limits<double>::infinity();
+        double in_value = cpl::NumericLimits<double>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
@@ -670,35 +806,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float32 to UInt64
     {
-        float in_value = std::numeric_limits<float>::quiet_NaN();
+        float in_value = cpl::NumericLimits<float>::quiet_NaN();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::infinity();
+        float in_value = -cpl::NumericLimits<float>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::max();
+        float in_value = -cpl::NumericLimits<float>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = std::numeric_limits<float>::max();
+        float in_value = cpl::NumericLimits<float>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
     }
 
     {
-        float in_value = std::numeric_limits<float>::infinity();
+        float in_value = cpl::NumericLimits<float>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
@@ -706,35 +842,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float64 to UInt64
     {
-        double in_value = -std::numeric_limits<double>::quiet_NaN();
+        double in_value = -cpl::NumericLimits<double>::quiet_NaN();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::infinity();
+        double in_value = -cpl::NumericLimits<double>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::max();
+        double in_value = -cpl::NumericLimits<double>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = std::numeric_limits<double>::max();
+        double in_value = cpl::NumericLimits<double>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
     }
 
     {
-        double in_value = std::numeric_limits<double>::infinity();
+        double in_value = cpl::NumericLimits<double>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
@@ -859,6 +995,49 @@ TEST_F(TestCopyWords, GDT_CFloat32and64)
     }
 }
 
+TEST_F(TestCopyWords, GDT_CFloat16only)
+{
+    GDALDataType intype = GDT_CFloat16;
+    for (GDALDataType outtype = GDT_Byte; outtype < GDT_TypeCount;
+         outtype = (GDALDataType)(outtype + 1))
+    {
+        if (IS_FLOAT(outtype))
+        {
+            FROM_C_F(intype, 127.1, 127.9, outtype, 127.1, 127.9);
+            FROM_C_F(intype, -127.1, -127.9, outtype, -127.1, -127.9);
+        }
+        else
+        {
+            FROM_C_F(intype, 126.1, 150.9, outtype, 126, 151);
+            FROM_C_F(intype, 126.9, 150.1, outtype, 127, 150);
+            if (!IS_UNSIGNED(outtype))
+            {
+                FROM_C_F(intype, -125.9, -127.1, outtype, -126, -127);
+            }
+        }
+    }
+    FROM_C(intype, -1, 256, GDT_Byte, 0, 0);
+    FROM_C(intype, 256, -1, GDT_Byte, 255, 0);
+    FROM_C(intype, -33000, 33000, GDT_Int16, -32768, 0);
+    FROM_C(intype, 33000, -33000, GDT_Int16, 32767, 0);
+    FROM_C(intype, -1, 66000, GDT_UInt16, 0, 0);
+    FROM_C(intype, 66000, -1, GDT_UInt16, 65535, 0);
+    FROM_C(intype, -33000, -33000, GDT_Int32, -32992, 0);
+    FROM_C(intype, 33000, 33000, GDT_Int32, 32992, 0);
+    FROM_C(intype, -1, 33000, GDT_UInt32, 0, 0);
+    FROM_C(intype, 33000, -1, GDT_UInt32, 32992, 0);
+    FROM_C(intype, 33000, -1, GDT_Float32, 32992, 0);
+    FROM_C(intype, 33000, -1, GDT_Float64, 32992, 0);
+    FROM_C(intype, -33000, -1, GDT_Float32, -32992, 0);
+    FROM_C(intype, -33000, -1, GDT_Float64, -32992, 0);
+    FROM_C(intype, -33000, 33000, GDT_CInt16, -32768, 32767);
+    FROM_C(intype, 33000, -33000, GDT_CInt16, 32767, -32768);
+    FROM_C(intype, -33000, -33000, GDT_CInt32, -32992, -32992);
+    FROM_C(intype, 33000, 33000, GDT_CInt32, 32992, 32992);
+    FROM_C(intype, 33000, -33000, GDT_CFloat32, 32992, -32992);
+    FROM_C(intype, 33000, -33000, GDT_CFloat64, 32992, -32992);
+}
+
 template <class Tin, class Tout>
 void CheckPackedGeneric(GDALDataType eIn, GDALDataType eOut)
 {
@@ -962,6 +1141,9 @@ template <class Tin> void CheckPacked(GDALDataType eIn, GDALDataType eOut)
         case GDT_Int64:
             CheckPacked<Tin, std::int64_t>(eIn, eOut);
             break;
+        case GDT_Float16:
+            CheckPacked<Tin, GFloat16>(eIn, eOut);
+            break;
         case GDT_Float32:
             CheckPacked<Tin, float>(eIn, eOut);
             break;
@@ -1001,6 +1183,9 @@ static void CheckPacked(GDALDataType eIn, GDALDataType eOut)
         case GDT_Int64:
             CheckPacked<std::int64_t>(eIn, eOut);
             break;
+        case GDT_Float16:
+            CheckPacked<GFloat16>(eIn, eOut);
+            break;
         case GDT_Float32:
             CheckPacked<float>(eIn, eOut);
             break;
