@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test core numeric operations and statistics calculations
@@ -31,6 +30,7 @@ import math
 import os
 import shutil
 import struct
+import sys
 
 import gdaltest
 import pytest
@@ -99,7 +99,7 @@ def test_stats_dont_force():
         gdal.Unlink("data/byte.tif.aux.xml")
     ds = gdal.Open("data/byte.tif")
     stats = ds.GetRasterBand(1).GetStatistics(0, 0)
-    assert stats == [0, 0, 0, -1], "did not get expected stats"
+    assert stats is None
 
 
 ###############################################################################
@@ -130,7 +130,7 @@ def test_stats_approx_nodata():
     os.remove("tmp/minfloat.tif")
 
     if nodata != -3.4028234663852886e38:
-        print("%.18g" % nodata)
+        print("%.17g" % nodata)
         pytest.fail("did not get expected nodata")
 
     assert stats == [-3.0, 5.0, 1.0, 4.0], "did not get expected stats"
@@ -180,6 +180,10 @@ def test_stats_nan_3():
 # and complex source nodata (#3576)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_stats_nan_4():
 
     ds = gdal.Open("data/nan32_nodata.vrt")
@@ -197,6 +201,10 @@ def test_stats_nan_4():
 # and complex source nodata (nan must be translated to 0 then) (#3576)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_stats_nan_5():
 
     ds = gdal.Open("data/nan32_nodata_nan_to_zero.vrt")
@@ -213,6 +221,10 @@ def test_stats_nan_5():
 # Test reading a warped VRT with nan as src nodata and dest nodata (#3576)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_stats_nan_6():
 
     ds = gdal.Open("data/nan32_nodata_warp.vrt")
@@ -229,6 +241,10 @@ def test_stats_nan_6():
 # Test reading a warped VRT with nan as src nodata and 0 as dest nodata (#3576)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_stats_nan_7():
 
     ds = gdal.Open("data/nan32_nodata_warp_nan_to_zero.vrt")
@@ -245,6 +261,10 @@ def test_stats_nan_7():
 # Test reading a warped VRT with zero as src nodata and nan as dest nodata (#3576)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_stats_nan_8():
 
     ds = gdal.Open("data/nan32_nodata_warp_zero_to_nan.vrt")
@@ -266,6 +286,7 @@ def stats_nodata_inf_progress_cbk(value, string, extra):
     extra[0] = value
 
 
+@pytest.mark.require_driver("HFA")
 def test_stats_nodata_inf():
 
     ds = gdal.GetDriverByName("HFA").Create(
@@ -411,7 +432,7 @@ def test_stats_flt_min():
     os.remove("tmp/flt_min.tif")
 
     if nodata != 1.17549435082228751e-38:
-        print("%.18g" % nodata)
+        print("%.17g" % nodata)
         pytest.fail("did not get expected nodata")
 
     assert stats == [0.0, 1.0, 0.33333333333333337, 0.47140452079103168] or stats == [
@@ -450,7 +471,7 @@ def test_stats_dbl_min():
     os.remove("tmp/dbl_min.tif")
 
     if nodata != 2.22507385850720138e-308:
-        print("%.18g" % nodata)
+        print("%.17g" % nodata)
         pytest.fail("did not get expected nodata")
 
     assert stats == [0.0, 1.0, 0.33333333333333337, 0.47140452079103168] or stats == [
@@ -762,7 +783,7 @@ def test_stats_approx_stats_flag(dt=gdal.GDT_Byte, struct_frmt="B"):
     approx_ok = 0
     force = 0
     stats = ds.GetRasterBand(1).GetStatistics(approx_ok, force)
-    assert stats == [0.0, 0.0, 0.0, -1.0], "did not get expected stats"
+    assert stats is None
 
     approx_ok = 0
     force = 1
@@ -789,12 +810,13 @@ def test_stats_all_nodata():
     with pytest.raises(Exception):
         ds.GetRasterBand(1).ComputeRasterMinMax()
 
-    with pytest.raises(Exception):
-        ds.GetRasterBand(1).ComputeRasterMinMax(can_return_none=True)
+    assert ds.GetRasterBand(1).ComputeRasterMinMax(can_return_none=True) is None
 
-    with pytest.raises(Exception):
-        # can_return_null also accepted for similarity with other methods
-        ds.GetRasterBand(1).ComputeRasterMinMax(can_return_null=True)
+    with gdaltest.disable_exceptions(), gdal.quiet_errors():
+        assert ds.GetRasterBand(1).ComputeRasterMinMax(can_return_none=True) is None
+
+    # can_return_null also accepted for similarity with other methods
+    assert ds.GetRasterBand(1).ComputeRasterMinMax(can_return_null=True) is None
 
     approx_ok = 1
     force = 1
@@ -824,15 +846,15 @@ def test_stats_clear():
     filename = "/vsimem/out.tif"
     gdal.Translate(filename, "data/byte.tif")
     ds = gdal.Open(filename)
-    assert ds.GetRasterBand(1).GetStatistics(False, False) == [0, 0, 0, -1]
-    assert ds.GetRasterBand(1).ComputeStatistics(False) != [0, 0, 0, -1]
+    assert ds.GetRasterBand(1).GetStatistics(False, False) is None
+    assert ds.GetRasterBand(1).ComputeStatistics(False) is not None
 
     ds = gdal.Open(filename)
-    assert ds.GetRasterBand(1).GetStatistics(False, False) != [0, 0, 0, -1]
+    assert ds.GetRasterBand(1).GetStatistics(False, False) is not None
     ds.ClearStatistics()
 
     ds = gdal.Open(filename)
-    assert ds.GetRasterBand(1).GetStatistics(False, False) == [0, 0, 0, -1]
+    assert ds.GetRasterBand(1).GetStatistics(False, False) is None
 
     gdal.GetDriverByName("GTiff").Delete(filename)
 
@@ -846,8 +868,8 @@ def test_stats_clear():
         (gdal.GDT_Int16, -32767, 32766),
         (gdal.GDT_UInt32, 1, (1 << 32) - 2),
         (gdal.GDT_Int32, -(1 << 31) + 1, (1 << 31) - 2),
-        (gdal.GDT_UInt64, 1, (1 << 53) - 2),
-        (gdal.GDT_Int64, -(1 << 53) + 2, (1 << 53) - 2),
+        (gdal.GDT_UInt64, 1, (1 << 60) - 1),
+        (gdal.GDT_Int64, -(1 << 60) - 1, (1 << 60) - 1),
         (
             gdal.GDT_Float32,
             -struct.unpack("f", struct.pack("f", 1e20))[0],
@@ -888,7 +910,12 @@ def test_stats_computeminmax(datatype, minval, maxval, nodata):
         buf_xsize=2,
         buf_ysize=1,
     )
-    assert ds.GetRasterBand(1).ComputeRasterMinMax(0) == (expected_minval, maxval)
+    if datatype in (gdal.GDT_Int64, gdal.GDT_UInt64):
+        assert ds.GetRasterBand(1).ComputeRasterMinMax(0) == pytest.approx(
+            (expected_minval, maxval), rel=1e-17
+        )
+    else:
+        assert ds.GetRasterBand(1).ComputeRasterMinMax(0) == (expected_minval, maxval)
 
 
 ###############################################################################
@@ -905,3 +932,52 @@ def test_stats_mask_band():
     assert src_ds.GetRasterBand(1).ComputeRasterMinMax(False) == (2, 3)
     assert src_ds.GetRasterBand(1).ComputeStatistics(False) == [2, 3, 2.5, 0.5]
     assert src_ds.GetRasterBand(1).GetHistogram(False) == [0, 0, 1, 1] + ([0] * 252)
+
+
+###############################################################################
+# Test statistics on a band with all values to large values
+
+
+@pytest.mark.parametrize(
+    "value", [sys.float_info.max, -sys.float_info.max, float("inf"), -float("inf")]
+)
+def test_stats_all_large_values(value):
+
+    src_ds = gdal.GetDriverByName("MEM").Create("", 2, 1, 1, gdal.GDT_Float64)
+    src_ds.WriteRaster(0, 0, 2, 1, struct.pack("d" * 2, value, value))
+    assert src_ds.GetRasterBand(1).ComputeRasterMinMax(False) == (value, value)
+    assert src_ds.GetRasterBand(1).ComputeStatistics(False) == [value, value, value, 0]
+
+
+###############################################################################
+
+
+def test_stats_int64():
+
+    src_ds = gdal.GetDriverByName("MEM").Create("", 2, 1, 1, gdal.GDT_Int64)
+    src_ds.GetRasterBand(1).SetNoDataValue(-(1 << 60) - 1)
+    src_ds.WriteRaster(0, 0, 2, 1, struct.pack("q" * 2, 1 << 50, -(1 << 60) - 1))
+    assert src_ds.GetRasterBand(1).ComputeRasterMinMax(False) == (1 << 50, 1 << 50)
+    assert src_ds.GetRasterBand(1).ComputeStatistics(False) == [
+        1 << 50,
+        1 << 50,
+        1 << 50,
+        0,
+    ]
+
+
+###############################################################################
+
+
+def test_stats_uint64():
+
+    src_ds = gdal.GetDriverByName("MEM").Create("", 2, 1, 1, gdal.GDT_UInt64)
+    src_ds.GetRasterBand(1).SetNoDataValue((1 << 60) - 1)
+    src_ds.WriteRaster(0, 0, 2, 1, struct.pack("Q" * 2, 1 << 50, (1 << 60) - 1))
+    assert src_ds.GetRasterBand(1).ComputeRasterMinMax(False) == (1 << 50, 1 << 50)
+    assert src_ds.GetRasterBand(1).ComputeStatistics(False) == [
+        1 << 50,
+        1 << 50,
+        1 << 50,
+        0,
+    ]
