@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test read/write functionality for ECW driver.
@@ -11,23 +10,7 @@
 # Copyright (c) 2003, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2009-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import array
@@ -111,6 +94,10 @@ def startup_and_cleanup():
     else:
         gdaltest.ecw_drv.major_version = 3
         gdaltest.ecw_drv.minor_version = 3
+    gdaltest.ecw_drv.version = (
+        gdaltest.ecw_drv.major_version,
+        gdaltest.ecw_drv.minor_version,
+    )
 
     # we set ECW to not resolve projection and datum strings to get 3.x behavior.
     with gdal.config_option("ECW_DO_NOT_RESOLVE_DATUM_PROJECTION", "YES"):
@@ -1038,6 +1025,10 @@ def test_ecw_31():
 # It ignores the content of panBandMap. (#4234)
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_ecw_32():
 
     ds = gdal.Open("data/ecw/jrc.ecw")
@@ -1479,7 +1470,7 @@ def test_ecw_41(tmp_path):
     # Check that no statistics is already included in the file
     assert ds.GetRasterBand(1).GetMinimum() is None
     assert ds.GetRasterBand(1).GetMaximum() is None
-    assert ds.GetRasterBand(1).GetStatistics(1, 0) == [0.0, 0.0, 0.0, -1.0]
+    assert ds.GetRasterBand(1).GetStatistics(1, 0) is None
     assert ds.GetRasterBand(1).GetDefaultHistogram(force=0) is None
 
     # Now compute the stats
@@ -2477,3 +2468,16 @@ def test_ecw_online_7():
         expected_band_count,
         ds.RasterCount,
     )
+
+
+###############################################################################
+# Test reading a dataset whose tile dimensions are larger than dataset ones
+
+
+def test_ecw_byte_tile_2048():
+
+    ds = gdal.Open("data/jpeg2000/byte_tile_2048.jp2")
+    (blockxsize, blockysize) = ds.GetRasterBand(1).GetBlockSize()
+    assert (blockxsize, blockysize) == (
+        (20, 20) if gdaltest.ecw_drv.version >= (5, 1) else (256, 256)
+    )
