@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test functionality for OGR Arrow driver.
@@ -10,23 +9,7 @@
 ###############################################################################
 # Copyright (c) 2022, Planet Labs
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import json
@@ -569,7 +552,7 @@ def test_ogr_arrow_read_arrow_json_extension():
     stream = lyr.GetArrowStream()
     schema = stream.GetSchema()
 
-    dst_ds = gdal.GetDriverByName("Memory").Create("", 0, 0, 0, gdal.GDT_Unknown)
+    dst_ds = gdal.GetDriverByName("MEM").Create("", 0, 0, 0, gdal.GDT_Unknown)
     dst_lyr = dst_ds.CreateLayer("test")
     success, error_msg = dst_lyr.IsArrowSchemaSupported(schema)
     assert success
@@ -738,38 +721,204 @@ def test_ogr_arrow_write_arrow_fid_in_input_but_not_in_output(tmp_vsimem):
 
 
 @gdaltest.enable_exceptions()
-def test_ogr_arrow_write_arrow_fid_in_output_but_not_in_input(tmp_vsimem):
+def test_ogr_arrow_ipc_read_stdin(tmp_path):
 
-    src_ds = ogr.Open("data/poly.shp")
-    src_lyr = src_ds.GetLayer(0)
-
-    outfilename = str(tmp_vsimem / "poly.feather")
-    with ogr.GetDriverByName("Arrow").CreateDataSource(outfilename) as dst_ds:
-        dst_lyr = dst_ds.CreateLayer(
+    outfilename = str(tmp_path / "poly.bin")
+    with ogr.GetDriverByName("Arrow").CreateDataSource(outfilename) as ds:
+        lyr = ds.CreateLayer(
             "test",
-            srs=src_lyr.GetSpatialRef(),
-            geom_type=ogr.wkbPoint,
-            options=["GEOMETRY_ENCODING=WKB", "FID=my_fid"],
+            geom_type=ogr.wkbNone,
+            options=["FORMAT=STREAM"],
+        )
+        fld_defn = ogr.FieldDefn("foo")
+        fld_defn.SetComment("x" * (1024 * 1024))
+        lyr.CreateField(fld_defn)
+        f = ogr.Feature(lyr.GetLayerDefn())
+        f["foo"] = "bar"
+        lyr.CreateFeature(f)
+
+    assert gdal.VSIStatL(outfilename).size > 1024 * 1024
+
+    # By default, as the header section is larger than 1 MB, we can't
+    # identify /vsistdin/
+    with gdaltest.config_options(
+        {
+            "CPL_VSISTDIN_FILE": outfilename,
+            "CPL_VSISTDIN_RESET_POSITION": "YES",
+            "CPL_VSISTDIN_FILE_CLOSE": "YES",
+        }
+    ):
+        with pytest.raises(Exception):
+            gdal.Open("/vsistdin/")
+
+    with gdaltest.config_options(
+        {
+            "CPL_VSISTDIN_FILE": outfilename,
+            "CPL_VSISTDIN_RESET_POSITION": "YES",
+            "CPL_VSISTDIN_FILE_CLOSE": "YES",
+        }
+    ):
+        ds = gdal.OpenEx("/vsistdin/", allowed_drivers=["ARROW"])
+        lyr = ds.GetLayer(0)
+        f = lyr.GetNextFeature()
+        assert f["foo"] == "bar"
+
+
+###############################################################################
+
+
+@gdaltest.enable_exceptions()
+def test_ogr_arrow_vsi_arrow_file_system():
+
+    version = int(
+        ogr.GetDriverByName("ARROW").GetMetadataItem("ARROW_VERSION").split(".")[0]
+    )
+    if version < 16:
+        pytest.skip("requires Arrow >= 16.0.0")
+
+    ogr.Open("gdalvsi://data/arrow/test.feather")
+
+
+###############################################################################
+
+
+@gdaltest.enable_exceptions()
+def test_ogr_arrow_string_view():
+
+    version = int(
+        ogr.GetDriverByName("ARROW").GetMetadataItem("ARROW_VERSION").split(".")[0]
+    )
+    if version < 15:
+        pytest.skip("requires Arrow >= 15")
+
+    with ogr.Open("data/arrow/stringview.feather") as ds:
+        lyr = ds.GetLayer(0)
+        f = lyr.GetNextFeature()
+        assert f["stringview"] == "foo"
+        assert f["list_stringview"] is None
+        assert f["list_of_list_stringview"] is None
+        assert f["map_stringview"] is None
+
+        f = lyr.GetNextFeature()
+        assert f["stringview"] == "bar"
+        assert f["list_stringview"] == [""]
+        assert f["list_of_list_stringview"] == "[null]"
+        assert f["map_stringview"] == "{}"
+
+        f = lyr.GetNextFeature()
+        assert f["stringview"] == "looooooooooong string"
+        assert f["list_stringview"] == ["foo", "bar", "looooooooooong string"]
+        assert f["list_of_list_stringview"] == '[["foo","bar","looooooooooong string"]]'
+        assert f["map_stringview"] == '{"x":"x_val","y":null}'
+
+
+###############################################################################
+
+
+@gdaltest.enable_exceptions()
+def test_ogr_arrow_binary_view():
+
+    version = int(
+        ogr.GetDriverByName("ARROW").GetMetadataItem("ARROW_VERSION").split(".")[0]
+    )
+    if version < 15:
+        pytest.skip("requires Arrow >= 15")
+
+    with ogr.Open("data/arrow/binaryview.feather") as ds:
+        lyr = ds.GetLayer(0)
+        f = lyr.GetNextFeature()
+        assert f.GetFieldAsBinary("binaryview") == b"foo"
+        f = lyr.GetNextFeature()
+        assert f.GetFieldAsBinary("binaryview") == b"bar"
+        f = lyr.GetNextFeature()
+        assert f.GetFieldAsBinary("binaryview") == b"looooooooooong binary"
+
+
+###############################################################################
+
+
+@gdaltest.enable_exceptions()
+@pytest.mark.parametrize(
+    "geom_type,encoding,expected_arrow_type",
+    [
+        (
+            ogr.wkbPoint,
+            "GEOARROW_STRUCT",
+            "struct<x: double not null, y: double not null>",
+        ),
+        (
+            ogr.wkbPoint,
+            "GEOARROW_INTERLEAVED",
+            "fixed_size_list<xy: double not null>[2]",
+        ),
+        (
+            ogr.wkbLineString,
+            "GEOARROW_STRUCT",
+            "list<vertices: struct<x: double not null, y: double not null>>",
+        ),
+        (
+            ogr.wkbLineString,
+            "GEOARROW_INTERLEAVED",
+            "list<vertices: fixed_size_list<xy: double not null>[2]>",
+        ),
+        (
+            ogr.wkbPolygon,
+            "GEOARROW_STRUCT",
+            "list<rings: list<vertices: struct<x: double not null, y: double not null>>>",
+        ),
+        (
+            ogr.wkbPolygon,
+            "GEOARROW_INTERLEAVED",
+            "list<rings: list<vertices: fixed_size_list<xy: double not null>[2]>>",
+        ),
+        (
+            ogr.wkbMultiPoint,
+            "GEOARROW_STRUCT",
+            "list<points: struct<x: double not null, y: double not null>>",
+        ),
+        (
+            ogr.wkbMultiPoint,
+            "GEOARROW_INTERLEAVED",
+            "list<points: fixed_size_list<xy: double not null>[2]>",
+        ),
+        (
+            ogr.wkbMultiLineString,
+            "GEOARROW_STRUCT",
+            "list<linestrings: list<vertices: struct<x: double not null, y: double not null>>>",
+        ),
+        (
+            ogr.wkbMultiLineString,
+            "GEOARROW_INTERLEAVED",
+            "list<linestrings: list<vertices: fixed_size_list<xy: double not null>[2]>>",
+        ),
+        (
+            ogr.wkbMultiPolygon,
+            "GEOARROW_STRUCT",
+            "list<polygons: list<rings: list<vertices: struct<x: double not null, y: double not null>>>>",
+        ),
+        (
+            ogr.wkbMultiPolygon,
+            "GEOARROW_INTERLEAVED",
+            "list<polygons: list<rings: list<vertices: fixed_size_list<xy: double not null>[2]>>>",
+        ),
+    ],
+)
+def test_ogr_arrow_check_geoarrow_types(
+    tmp_path, geom_type, encoding, expected_arrow_type
+):
+    pytest.importorskip("pyarrow")
+    import pyarrow.feather
+
+    filename = str(tmp_path / "out.feather")
+    srs = osr.SpatialReference()
+    srs.ImportFromEPSG(4326)
+    with ogr.GetDriverByName("ARROW").CreateDataSource(filename) as ds:
+        ds.CreateLayer(
+            "test",
+            geom_type=geom_type,
+            srs=srs,
+            options=["GEOMETRY_ENCODING=" + encoding],
         )
 
-        stream = src_lyr.GetArrowStream(["INCLUDE_FID=NO"])
-        schema = stream.GetSchema()
-
-        success, error_msg = dst_lyr.IsArrowSchemaSupported(schema)
-        assert success
-
-        for i in range(schema.GetChildrenCount()):
-            if schema.GetChild(i).GetName() not in ("wkb_geometry", "OGC_FID"):
-                dst_lyr.CreateFieldFromArrowSchema(schema.GetChild(i))
-
-        while True:
-            array = stream.GetNextRecordBatch()
-            if array is None:
-                break
-            assert dst_lyr.WriteArrowBatch(schema, array) == ogr.OGRERR_NONE
-
-    ds = ogr.Open(outfilename)
-    lyr = ds.GetLayer(0)
-    src_lyr.ResetReading()
-    for i in range(src_lyr.GetFeatureCount()):
-        assert str(src_lyr.GetNextFeature()) == str(lyr.GetNextFeature())
+    schema = pyarrow.feather.read_table(filename).schema
+    assert str(schema.field("geometry").type) == expected_arrow_type
