@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test basic OGR functionality against test shapefiles.
@@ -46,6 +45,8 @@ def test_ogr_basic_1():
 
     assert ds is not None
 
+    assert isinstance(ds, ogr.DataSource)
+
 
 ###############################################################################
 # Test Feature counting.
@@ -447,8 +448,11 @@ def test_ogr_basic_10():
     if test_cli_utilities.get_test_ogrsf_path() is None:
         pytest.skip()
 
+    # --config OPENFILEGDB_REPRODUCIBLE_UUID=YES helps avoiding unsigned-integer-overflow
+    # under UBSAN.
     ret = gdaltest.runexternal(
-        test_cli_utilities.get_test_ogrsf_path() + " -all_drivers"
+        test_cli_utilities.get_test_ogrsf_path()
+        + " -all_drivers --config OPENFILEGDB_REPRODUCIBLE_UUID=YES"
     )
 
     assert "INFO" in ret
@@ -686,7 +690,7 @@ def test_ogr_basic_invalid_unicode():
     except Exception:
         pass
 
-    data_source = ogr.GetDriverByName("Memory").CreateDataSource("")
+    data_source = ogr.GetDriverByName("MEM").CreateDataSource("")
     layer = data_source.CreateLayer("test")
     layer.CreateField(ogr.FieldDefn("attr", ogr.OFTString))
     feature = ogr.Feature(layer.GetLayerDefn())
@@ -698,7 +702,7 @@ def test_ogr_basic_invalid_unicode():
 
 def test_ogr_basic_dataset_slice():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     ds.CreateLayer("lyr1")
     ds.CreateLayer("lyr2")
     ds.CreateLayer("lyr3")
@@ -713,6 +717,36 @@ def test_ogr_basic_dataset_slice():
     assert lyrs == ["lyr1", "lyr3"]
 
 
+def test_ogr_basic_dataset_iter():
+
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
+    ds.CreateLayer("lyr1")
+    ds.CreateLayer("lyr2")
+    ds.CreateLayer("lyr3")
+
+    layers = []
+
+    assert len(ds) == 3
+
+    for lyr in ds:
+        layers.append(lyr)
+
+    assert len(layers) == 3
+
+
+def test_ogr_basic_dataset_getitem():
+
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
+    ds.CreateLayer("lyr1")
+    ds.CreateLayer("lyr2")
+    ds.CreateLayer("lyr3")
+
+    assert ds[2].this == ds.GetLayer(2).this
+
+    with pytest.raises(IndexError):
+        ds[3]
+
+
 def test_ogr_basic_feature_iterator():
 
     ds = ogr.Open("data/poly.shp")
@@ -731,7 +765,7 @@ def test_ogr_basic_feature_iterator():
 
 def test_ogr_basic_dataset_copy_layer_dst_srswkt():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     src_lyr = ds.CreateLayer("lyr1")
     sr = osr.SpatialReference()
     sr.SetFromUserInput("WGS84")
@@ -743,7 +777,7 @@ def test_ogr_basic_dataset_copy_layer_dst_srswkt():
 
 def test_ogr_basic_dataset_copy_layer_metadata():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     src_lyr = ds.CreateLayer("lyr1")
     src_lyr.SetMetadataItem("foo", "bar")
     out_lyr = ds.CopyLayer(src_lyr, "lyr2")
@@ -752,7 +786,7 @@ def test_ogr_basic_dataset_copy_layer_metadata():
 
 def test_ogr_basic_dataset_no_copy_layer_metadata():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     src_lyr = ds.CreateLayer("lyr1")
     src_lyr.SetMetadataItem("foo", "bar")
     out_lyr = ds.CopyLayer(src_lyr, "lyr2", options=["COPY_MD=NO"])
@@ -800,7 +834,7 @@ def test_ogr_basic_float32_formatting():
 def test_ogr_basic_get_geometry_types():
     """Test Layer.GetGeometryTypes()"""
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("layer")
     lyr.CreateField(ogr.FieldDefn("str", ogr.OFTString))
     lyr.CreateField(ogr.FieldDefn("str2", ogr.OFTString))
@@ -995,6 +1029,7 @@ def test_datasource_use_after_close_2():
         ds2.GetLayer(0)
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_datasource_use_after_destroy():
     ds = ogr.Open("data/poly.shp")
 
@@ -1009,6 +1044,7 @@ def test_datasource_use_after_destroy():
         ds2.GetLayer(0)
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_datasource_use_after_release():
     ds = ogr.Open("data/poly.shp")
 
@@ -1060,6 +1096,7 @@ def test_layer_use_after_datasource_close_3(tmp_path):
         lyr2.GetFeatureCount()
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_layer_use_after_datasource_destroy():
     ds = ogr.Open("data/poly.shp")
     lyr = ds.GetLayerByName("poly")
@@ -1071,6 +1108,7 @@ def test_layer_use_after_datasource_destroy():
         lyr.GetFeatureCount()
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_layer_use_after_datasource_release():
     ds = ogr.Open("data/poly.shp")
     lyr = ds.GetLayerByName("poly")
@@ -1175,3 +1213,27 @@ def test_general_cmd_line_processor(tmp_path):
         ["program", 2, tmp_path / "a_path", "a_string"]
     )
     assert processed == ["program", "2", str(tmp_path / "a_path"), "a_string"]
+
+
+def test_driver_open_throw_1():
+
+    with gdaltest.enable_exceptions():
+        drv = ogr.GetDriverByName("ESRI Shapefile")
+
+        assert isinstance(drv, ogr.Driver)
+
+        with pytest.raises(RuntimeError, match="No such file or directory"):
+            drv.Open("does_not_exist.shp")
+
+
+def test_driver_open_throw_2():
+
+    with gdaltest.enable_exceptions():
+        drv = ogr.GetDriverByName("MapInfo File")
+        if not drv:
+            pytest.skip("MapInfo driver not available")
+
+        assert isinstance(drv, ogr.Driver)
+
+        with pytest.raises(RuntimeError, match="not recognized"):
+            drv.Open("data/poly.shp")
