@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test MySQL driver functionality.
@@ -350,8 +349,6 @@ def test_ogr_mysql_6(mysql_ds):
             max_error=1e-3,
         )
 
-        feat_read.Destroy()
-
         sql_lyr.ResetReading()
 
         with ogrtest.spatial_filter(sql_lyr, "LINESTRING(-10 -10,0 0)"):
@@ -556,7 +553,6 @@ def test_ogr_mysql_20(mysql_ds):
     # We are obliged to create a fake geometry
     dst_feat.SetGeometryDirectly(ogr.CreateGeometryFromWkt("POINT(0 1)"))
     layer.CreateFeature(dst_feat)
-    dst_feat.Destroy()
 
     layer = mysql_ds.GetLayerByName("select")
     layer.ResetReading()
@@ -602,8 +598,6 @@ def test_ogr_mysql_22(mysql_ds):
 
     layer.CreateFeature(dst_feat)
 
-    dst_feat.Destroy()
-
     layer.ResetReading()
     feat = layer.GetNextFeature()
     assert feat is not None
