@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test OGR SQL capabilities added as part of RFC 28 implementation.
@@ -11,23 +10,7 @@
 # Copyright (c) 2010, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2010-2014, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 
@@ -414,7 +397,7 @@ def test_ogr_rfc28_19(data_ds):
 
 def test_ogr_rfc28_20():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("my_ds")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("my_ds")
     lyr = ds.CreateLayer("my_layer")
     field_defn = ogr.FieldDefn("intfield", ogr.OFTInteger)
     lyr.CreateField(field_defn)
@@ -880,12 +863,12 @@ def test_ogr_rfc28_39(data_ds):
 
 
 ###############################################################################
-# Test MIN(), MAX() and AVG() on a date (#5333)
+# Test MIN(), MAX(), AVG(), STDDEV_POP(), STDDEV_SAMP() on a date (#5333)
 
 
 def test_ogr_rfc28_40():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("test")
     lyr.CreateField(ogr.FieldDefn("DATE", ogr.OFTDateTime))
     feat = ogr.Feature(lyr.GetLayerDefn())
@@ -895,15 +878,16 @@ def test_ogr_rfc28_40():
     feat.SetField(0, "2013/01/01 00:00:00")
     lyr.CreateFeature(feat)
 
-    with ds.ExecuteSQL("SELECT MIN(DATE), MAX(DATE), AVG(DATE) from test") as lyr:
+    with ds.ExecuteSQL(
+        "SELECT MIN(DATE), MAX(DATE), AVG(DATE), STDDEV_POP(DATE), STDDEV_SAMP(DATE) from test"
+    ) as sql_lyr:
 
-        ogrtest.check_features_against_list(lyr, "MIN_DATE", ["2013/01/01 00:00:00"])
-        lyr.ResetReading()
-        ogrtest.check_features_against_list(lyr, "MAX_DATE", ["2013/12/31 23:59:59"])
-        lyr.ResetReading()
-        ogrtest.check_features_against_list(
-            lyr, "AVG_DATE", ["2013/07/02 11:59:59.500"]
-        )
+        f = sql_lyr.GetNextFeature()
+        assert f["MIN_DATE"] == "2013/01/01 00:00:00"
+        assert f["MAX_DATE"] == "2013/12/31 23:59:59"
+        assert f["AVG_DATE"] == "2013/07/02 11:59:59.500"
+        assert f["STDDEV_POP_DATE"] == pytest.approx(15767999.5, rel=1e-15)
+        assert f["STDDEV_SAMP_DATE"] == pytest.approx(22299318.744392183, rel=1e-15)
 
 
 ###############################################################################
@@ -912,7 +896,7 @@ def test_ogr_rfc28_40():
 
 def test_ogr_rfc28_41():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("my_ds")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("my_ds")
     lyr = ds.CreateLayer("my_layer")
     field_defn = ogr.FieldDefn("a.b", ogr.OFTInteger)
     lyr.CreateField(field_defn)
@@ -939,7 +923,7 @@ def test_ogr_rfc28_41():
 
 def test_ogr_rfc28_42():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("test")
     fld_defn = ogr.FieldDefn("b", ogr.OFTInteger)
     fld_defn.SetSubType(ogr.OFSTBoolean)
@@ -1007,7 +991,7 @@ def test_ogr_rfc28_42():
 
 def test_ogr_rfc28_43():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("test")
     fld_defn = ogr.FieldDefn("myint64", ogr.OFTInteger64)
     lyr.CreateField(fld_defn)
@@ -1065,7 +1049,7 @@ def test_ogr_rfc28_43():
 
 def test_ogr_rfc28_44():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr.withpoint")
     fld_defn = ogr.FieldDefn("field.withpoint", ogr.OFTInteger)
     lyr.CreateField(fld_defn)
@@ -1158,7 +1142,7 @@ def test_ogr_rfc28_45(data_ds):
 
 def test_ogr_rfc28_46():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr")
     fld_defn = ogr.FieldDefn("val", ogr.OFTInteger)
     lyr.CreateField(fld_defn)
@@ -1320,7 +1304,7 @@ def test_ogr_rfc28_47(data_ds):
 
 def test_ogr_rfc28_48():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr")
     fld_defn = ogr.FieldDefn("dt", ogr.OFTDateTime)
     lyr.CreateField(fld_defn)
@@ -1394,7 +1378,7 @@ def test_ogr_rfc28_48():
 ###############################################################################
 def test_ogr_rfc28_datetime_null():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr")
     fld_defn = ogr.FieldDefn("dt", ogr.OFTDateTime)
     lyr.CreateField(fld_defn)
@@ -1418,7 +1402,7 @@ def test_ogr_rfc28_datetime_null():
 ###############################################################################
 def test_ogr_rfc28_int_overflows():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr")
     f = ogr.Feature(lyr.GetLayerDefn())
     lyr.CreateFeature(f)
@@ -1471,7 +1455,7 @@ def test_ogr_rfc28_int_overflows():
 
 def test_ogr_rfc28_many_or():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr")
     fld_defn = ogr.FieldDefn("val", ogr.OFTInteger)
     lyr.CreateField(fld_defn)
@@ -1495,7 +1479,7 @@ def test_ogr_rfc28_many_or():
 
 def test_ogr_rfc28_many_and():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr")
     fld_defn = ogr.FieldDefn("val", ogr.OFTInteger)
     lyr.CreateField(fld_defn)
@@ -1520,7 +1504,7 @@ def test_ogr_rfc28_many_and():
 
 def test_ogr_rfc28_nested_or():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("test", geom_type=ogr.wkbNone)
     field = ogr.FieldDefn("fclass", ogr.OFTString)
     lyr.CreateField(field)
@@ -1573,7 +1557,7 @@ def test_ogr_rfc28_nested_or():
 
 def test_ogr_rfc28_order_by_two_columns():
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("lyr")
     fld_defn = ogr.FieldDefn("int_val", ogr.OFTInteger)
     lyr.CreateField(fld_defn)
@@ -1611,7 +1595,7 @@ def test_ogr_rfc28_order_by_two_columns():
 def test_ogr_rfc28_in_date_filter():
     """Test that date fields stored as ISO-8601 can be used with IN operator"""
 
-    ds = ogr.GetDriverByName("Memory").CreateDataSource("")
+    ds = ogr.GetDriverByName("MEM").CreateDataSource("")
     lyr = ds.CreateLayer("ogr_in_date_filter", geom_type=ogr.wkbNone)
     lyr.CreateField(ogr.FieldDefn("date_minus", ogr.OFTDate))
     lyr.CreateField(ogr.FieldDefn("date_slash", ogr.OFTDate))
