@@ -2136,9 +2136,12 @@ def error_raised(type, match=""):
     with error_handler(handler):
         yield
 
-    assert any(
-        [err["level"] == type and match in err["message"] for err in errors]
-    ), f'Did not receive an error of type {err_levels[type]} matching "{match}". Received: {[(err["level"], err["message"]) for err in errors]}'
+    if type == gdal.CE_None:
+        assert not any([err["level"] != gdal.CE_Debug for err in errors])
+    else:
+        assert any(
+            [err["level"] == type and match in err["message"] for err in errors]
+        ), f'Did not receive an error of type {err_levels[type]} matching "{match}". Received: {[(err["level"], err["message"]) for err in errors]}'
 
 
 ###############################################################################
