@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  gdal2tiles.py testing
@@ -10,23 +9,7 @@
 ###############################################################################
 # Copyright (c) 2015, Even Rouault <even dot rouault @ spatialys dot com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import glob
@@ -43,10 +26,15 @@ import test_py_scripts  # noqa  # pylint: disable=E0401
 from osgeo import gdal, osr  # noqa
 from osgeo_utils.gdalcompare import compare_db
 
-pytestmark = pytest.mark.skipif(
-    test_py_scripts.get_py_script("gdal2tiles") is None,
-    reason="gdal2tiles not available",
-)
+pytestmark = [
+    pytest.mark.skipif(
+        test_py_scripts.get_py_script("gdal2tiles") is None,
+        reason="gdal2tiles not available",
+    ),
+    pytest.mark.skipif(
+        not gdaltest.vrt_has_open_support(), reason="VRT driver open missing"
+    ),
+]
 
 
 @pytest.fixture()
@@ -60,6 +48,9 @@ def script_path():
 
 def test_gdal2tiles_help(script_path):
 
+    if gdaltest.is_travis_branch("sanitize"):
+        pytest.skip("fails on sanitize for unknown reason")
+
     assert "ERROR" not in test_py_scripts.run_py_script(
         script_path, "gdal2tiles", "--help"
     )
@@ -71,6 +62,9 @@ def test_gdal2tiles_help(script_path):
 
 def test_gdal2tiles_version(script_path):
 
+    if gdaltest.is_travis_branch("sanitize"):
+        pytest.skip("fails on sanitize for unknown reason")
+
     assert "ERROR" not in test_py_scripts.run_py_script(
         script_path, "gdal2tiles", "--version"
     )
@@ -644,10 +638,11 @@ def test_gdal2tiles_nodata_values_pct_threshold(script_path, tmp_path):
     output_folder = str(tmp_path / "test_gdal2tiles_nodata_values_pct_threshold")
 
     src_ds = gdal.GetDriverByName("GTiff").Create(input_tif, 256, 256, 3, gdal.GDT_Byte)
-    src_ds.GetRasterBand(1).SetNoDataValue(20)
-    src_ds.GetRasterBand(1).WriteRaster(
-        0, 0, 2, 2, struct.pack("B" * 4, 10, 20, 30, 40)
-    )
+    for i in range(3):
+        src_ds.GetRasterBand(i + 1).SetNoDataValue(20)
+        src_ds.GetRasterBand(i + 1).WriteRaster(
+            0, 0, 2, 2, struct.pack("B" * 4, 10, 20, 30, 40)
+        )
     srs = osr.SpatialReference()
     srs.ImportFromEPSG(3857)
     src_ds.SetSpatialRef(srs)
@@ -728,6 +723,8 @@ def test_gdal2tiles_py_jpeg_3band_input(
     script_path, tmp_path, resampling, expected_stats_z0, expected_stats_z1
 ):
 
+    gdaltest.importorskip_gdal_array()
+
     if resampling == "antialias" and not pil_available():
         pytest.skip("'antialias' resampling is not available")
 
@@ -792,6 +789,8 @@ def test_gdal2tiles_py_jpeg_1band_input(
     script_path, tmp_path, resampling, expected_stats_z14, expected_stats_z13
 ):
 
+    gdaltest.importorskip_gdal_array()
+
     if resampling == "antialias" and not pil_available():
         pytest.skip("'antialias' resampling is not available")
 
