@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  test librarified gdalmdimtranslate
@@ -10,26 +9,11 @@
 ###############################################################################
 # Copyright (c) 2019, Even Rouault <even.rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import collections
+import os
 import pathlib
 import struct
 
@@ -41,6 +25,10 @@ from osgeo import gdal
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_no_arg(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -52,6 +40,10 @@ def test_gdalmdimtranslate_no_arg(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_multidim_to_mem():
 
     out_ds = gdal.MultiDimTranslate("", "data/mdim.vrt", format="MEM")
@@ -66,6 +58,26 @@ def test_gdalmdimtranslate_multidim_to_mem():
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
+def test_gdalmdimtranslate_multidim_to_unknown_format():
+
+    with pytest.raises(
+        Exception,
+        match="Cannot determine output driver for dataset name 'unknown.unknown'",
+    ):
+        gdal.MultiDimTranslate("unknown.unknown", "data/mdim.vrt")
+
+
+###############################################################################
+
+
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_multidim_to_classic(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.tif"
@@ -83,6 +95,10 @@ def test_gdalmdimtranslate_multidim_to_classic(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_multidim_1d_to_classic(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.tif"
@@ -111,12 +127,16 @@ def test_gdalmdimtranslate_classic_to_classic(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_classic_to_multidim(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
     tmpgtifffile = tmp_vsimem / "tmp.tif"
-    if gdal.VSIStatL("../gcore/data/byte.tif.aux.xml"):
-        gdal.Unlink("../gcore/data/byte.tif.aux.xml")
+    if os.path.exists("../gcore/data/byte.tif.aux.xml"):
+        os.unlink("../gcore/data/byte.tif.aux.xml")
     ds = gdal.Translate(tmpgtifffile, "../gcore/data/byte.tif")
     ds.SetSpatialRef(None)
     ds = None
@@ -170,6 +190,10 @@ def test_gdalmdimtranslate_classic_to_multidim(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_array(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -251,6 +275,10 @@ def test_gdalmdimtranslate_array(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_array_with_transpose_and_view(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -326,6 +354,10 @@ def test_gdalmdimtranslate_array_with_transpose_and_view(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_group(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -391,6 +423,10 @@ def test_gdalmdimtranslate_group(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_two_groups(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -461,6 +497,10 @@ def test_gdalmdimtranslate_two_groups(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_subset(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -716,6 +756,10 @@ def test_gdalmdimtranslate_subset(tmp_vsimem):
 ###############################################################################
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_scaleaxes(tmp_vsimem):
 
     tmpfile = tmp_vsimem / "out.vrt"
@@ -790,6 +834,10 @@ def test_gdalmdimtranslate_scaleaxes(tmp_vsimem):
     )
 
 
+@pytest.mark.skipif(
+    not gdaltest.vrt_has_open_support(),
+    reason="VRT driver open missing",
+)
 def test_gdalmdimtranslate_dims_with_same_name_different_size(tmp_vsimem):
 
     srcfile = tmp_vsimem / "in.vrt"
